datablock ItemData(SkateboardItem)
{
	category = "Item";  // Mission editor category

	equipment = true;

	//its already a member of item namespace so dont break it
	//className = "Item"; // For inventory system

	 // Basic Item Properties
	shapeFile = "./skateboard_image.dts";
	mass = 1;
	density = 0.2;
	elasticity = 0.2;
	friction = 0.6;
	emap = true;

	//gui stuff
	uiName = "Skateboard";
	iconName = "./icon";
	doColorShift = true;
	colorShiftColor = "1.000 1.000 1.000 1.000";
	
	 // Dynamic properties defined by the scripts
	image = SkateboardImage;
	canDrop = true;
};
datablock ShapeBaseImageData(SkateboardImage)
{
   // Basic Item properties
   shapeFile = "./skateboard_image.dts";
   emap = true;

   // Specify mount point & offset for 3rd person, and eye offset
   // for first person rendering.
   mountPoint = 0;
   offset = "0.0 0.2 0.0";
   rotation = eulerToMatrix("90 90 0");
   eyeOffset = "0 0 0";
   eyeRotation = eulerToMatrix("0 0 0");

   doColorShift = true;
	colorShiftColor = SkateboardItem.colorShiftColor;
   

   // When firing from a point offset from the eye, muzzle correction
   // will adjust the muzzle vector to point to the eye LOS point.
   // Since this weapon doesn't actually fire from the muzzle point,
   // we need to turn this off.  
   correctMuzzleVector = false;

   // Add the WeaponImage namespace as a parent, WeaponImage namespace
   // provides some hooks into the inventory system.
   className = "WeaponImage";

   // Projectile && Ammo.
   item = SkateboardItem;

   //melee particles shoot from eye node for consistancy
   melee = true;
   //raise your arm up or not
   armReady = false;

   //casing = " ";

   doColorShift = true;
   colorShiftColor = SkateboardItem.colorShiftColor; //"0.200 0.200 0.200 1.000";

   // Images have a state system which controls how the animations
   // are run, which sounds are played, script callbacks, etc. This
   // state system is downloaded to the client so that clients can
   // predict state changes and animate accordingly.  The following
   // system supports basic ready->fire->reload transitions as
   // well as a no-ammo->dryfire idle state.

   // Initial start up state
	stateName[0]                     = "Activate";
	stateTimeoutValue[0]             = 0.5;
	stateTransitionOnTimeout[0]       = "Ready";

	stateName[1]                     = "Ready";
	stateTransitionOnTriggerDown[1]  = "Fire";
	stateAllowImageChange[1]         = true;

	stateName[2]                    = "Fire";
	stateTransitionOnTriggerUp[2]	= "Ready";
	stateScript[2]                  = "onFire";
};

function SkateboardImage::onMount(%this, %obj, %slot)
{
	fixArmReady(%obj);
	return;
}

function SkateboardImage::onFire(%this, %obj, %slot)
{
	%player = %obj;

	if(%player.isMounted())
	{
		%mount = %player.getObjectMount();
		%mountedVehicleName = %mount.getDataBlock().getName();
		if(%mountedVehicleName !$= "SkateboardPlayer")
		{	
			commandToClient(%player.client, 'CenterPrint', "\c4Can\'t use skateboard right now.", 2);
			return;
		}
		else
		{
			if(isObject(%mount.spawnBrick))
			{
				commandToClient(%player.client, 'CenterPrint', "\c4Can\'t use skateboard right now.", 2);
				return;
			}
			%mount.delete();
		}
	}
	else
	{
		%player.startSkateboarding();
		commandToClient(%player.client,'setScrollMode', -1);
		%player.unMountimage(%slot);
		fixArmReady(%player);
	}
}

function Player::startSkateboarding(%obj)
{
	//make a new skate vehicle and mount the player on it
	%client = %obj.client;
	%position = %obj.getTransform();
	%posX = getword(%position, 0);
	%posY = getword(%position, 1);
	%posZ = getword(%position, 2);
	%rot = getWords(%position, 3, 8);

	%posZ += 0.3;

	%vel = %obj.getVelocity();

	%newcar = new AIPlayer() 
	{
		dataBlock = SkateboardPlayer;
		client = %client;
		initialPosition = %posX @ " " @ %posY @ " " @ %posZ;
	};
	MissionCleanup.add(%newcar);
	%newcar.setVelocity(%vel);
	%newcar.setTransform(%posX @ " " @ %posY @ " " @ %posZ @ " " @ %rot);
	%newcar.schedule(10, mountObject, %obj, 0);
	%skateboard = %newcar;
	
	if(%client.prefSkateboardIndex > 0 && %client.prefSkateboardIndex <= $SkateboardMaxLogo)
		%newcar.skateboardSetLogo(%client.prefSkateboardIndex);
		
	if(%client.persLogoDamage > 0)
	{
		%newcar.logoDamage = %client.persLogoDamage;
		%newcar.skateboardUpdateDamage();
	}
	
	if(!%client.hasSetColorOnce)
	{
		%skateboard.skateboardSetDefaultColor();
	}
	else
	{
		%skateboard.doNotResetColor = true;
		if(%client.skateboardWheelColor 	$= ""){%client.skateboardWheelColor 	= $SkateboardDefaultWheelColor;}
		if(%client.skateboardTruckColor 	$= ""){%client.skateboardTruckColor		= $SkateboardDefaultTruckColor;}
		if(%client.skateboardGripColor 		$= ""){%client.skateboardGripColor		= $SkateboardDefaultGripColor;}
		if(%client.skateboardBoardColor 	$= ""){%client.skateboardBoardColor		= $SkateboardDefaultWoodColor;}
		if(%client.skateboardHardwareColor 	$= ""){%client.skateboardHardwareColor 	= $SkateboardDefaultHardwareColor;}
	
		%skateboard.skateboardSetWheelColor(%client.skateboardWheelColor);
		%skateboard.skateboardSetTruckColor(%client.skateboardTruckColor);
		%skateboard.skateboardSetGripTapeColor(%client.skateboardGripColor);
		%skateboard.skateboardSetBoardColor(%client.skateboardBoardColor);
		%skateboard.skateboardSetHardwareColor(%client.skateboardHardwareColor);
	}
}

function skateboardPlayer::onDriverLeave(%this, %obj, %player)
{
	if(isObject(%obj.spawnBrick))
		return;
	
	if(isObject(%player))
	{
		%player.setTransform(%obj.getTransform());
	}
	else
	{
		%obj.schedule(10, skateboardDoDeath);
		return;
	}

	if(%obj.isWrecked)
		%obj.schedule(10, skateboardDoDeath);
	else
	{
		%obj.hideNode("ALL");
		%obj.schedule(10, delete);
	}
}