datablock AudioProfile(SkateboardRollSound)
{
   filename    = "./sounds/roll.wav";
   description = AudioClosestLooping3d;
   preload = true;
};
datablock AudioProfile(SkateboardBrakeSound1)
{
   filename    = "./sounds/brake2.wav";
   description = AudioClosest3d;
   preload = true;
};
datablock AudioProfile(SkateboardBrakeSound2)
{
   filename    = "./sounds/brake3.wav";
   description = AudioClosest3d;
   preload = true;
};
datablock AudioProfile(SkateboardBrakeSound3)
{
   filename    = "./sounds/brake4.wav";
   description = AudioClosest3d;
   preload = true;
};

// global vars so we can quickly randomize these later
$SkateboardBrakeSound[1] = SkateboardBrakeSound1;
$SkateboardBrakeSound[2] = SkateboardBrakeSound2;
$SkateboardBrakeSound[3] = SkateboardBrakeSound3;

datablock AudioProfile(SkateboardGrindSound)
{
   filename    = "./sounds/grind.wav";
   description = AudioClosestLooping3d;
   preload = true;
};
datablock AudioProfile(SkateboardWheelSpinSound)
{
   filename    = "./sounds/wheelSpin.wav";
   description = AudioClosestLooping3d;
   preload = true;
};
datablock AudioProfile(SkateboardTrickSound)
{
   filename    = "./sounds/trick.wav";
   description = AudioClosest3d;
   preload = true;
};
datablock AudioProfile(SkateboardPopSound)
{
   filename    = "./sounds/pop.wav";
   description = AudioClose3d;
   preload = true;
};


// gay math
function eulerToAxis(%euler)
{
	%euler = VectorScale(%euler,$pi / 180);
	%matrix = MatrixCreateFromEuler(%euler);
	return getWords(%matrix,3,6);
}

function axisToEuler(%axis)
{
	%angleOver2 = getWord(%axis,3) * 0.5;
	%angleOver2 = -%angleOver2;
	%sinThetaOver2 = mSin(%angleOver2);
	%cosThetaOver2 = mCos(%angleOver2);
	%q0 = %cosThetaOver2;
	%q1 = getWord(%axis,0) * %sinThetaOver2;
	%q2 = getWord(%axis,1) * %sinThetaOver2;
	%q3 = getWord(%axis,2) * %sinThetaOver2;
	%q0q0 = %q0 * %q0;
	%q1q2 = %q1 * %q2;
	%q0q3 = %q0 * %q3;
	%q1q3 = %q1 * %q3;
	%q0q2 = %q0 * %q2;
	%q2q2 = %q2 * %q2;
	%q2q3 = %q2 * %q3;
	%q0q1 = %q0 * %q1;
	%q3q3 = %q3 * %q3;
	%m13 = 2.0 * (%q1q3 - %q0q2);
	%m21 = 2.0 * (%q1q2 - %q0q3);
	%m22 = 2.0 * %q0q0 - 1.0 + 2.0 * %q2q2;
	%m23 = 2.0 * (%q2q3 + %q0q1);
	%m33 = 2.0 * %q0q0 - 1.0 + 2.0 * %q3q3;
	return mRadToDeg(mAsin(%m23)) SPC mRadToDeg(mAtan(-%m13, %m33)) SPC mRadToDeg(mAtan(-%m21, %m22));
}

function Normal2Rotation(%normal)  
{  
   if(getWord(%normal, 2) == 1 || getWord(%normal, 2) == -1)  
      %rotAxis = vectorNormalize(vectorCross(%normal, "0 1 0"));  
   else  
      %rotAxis = vectorNormalize(vectorCross(%normal, "0 0 1"));  
  
   %initialAngle = mACos(vectorDot(%normal, "0 0 1"));  
   %rotation = %rotAxis SPC (%initialAngle);  
  
   return %rotation;  
}

// ripped from swollows sweps addon because i was too lazy to do it myself thanks swollow
datablock PlayerData(emptyPlayer : playerStandardArmor)
{
	shapeFile = "base/data/shapes/empty.dts";
	boundingBox = "0.01 0.01 0.01";
	crouchboundingBox = "0.01 0.01 0.01";
	isEmptyPlayer = true;
	deathSound = "";
	painSound = "";
	useCustomPainEffects = true;
	mountSound = "";
	jumpSound = "";
	uiName = "";
};

// debris for when we drop the board after a bail (or the skateboard breaks)
datablock DebrisData(skateboardDeathDebris)
{
   emitters = "";

	shapeFile = "./skateboard_blank.dts";
	lifetime = 3.0;
	minSpinSpeed = -300.0;
	maxSpinSpeed = 300.0;
	elasticity = 0.5;
	friction = 0.2;
	numBounces = 1;
	staticOnMaxBounce = true;
	snapOnMaxBounce = false;
	fade = true;

	gravModifier = 2;
};

datablock ExplosionData(skateboardDeathExplosion)
{
   //explosionShape = "";
   lifeTimeMS = 150;

   soundProfile = "";

   debris = skateboardDeathDebris;
   debrisNum = 1;
   debrisNumVariance = 0;
   debrisPhiMin = 0;
   debrisPhiMax = 360;
   debrisThetaMin = 0;
   debrisThetaMax = 20;
   debrisVelocity = 9;
   debrisVelocityVariance = 8;

   faceViewer     = true;
   explosionScale = "1 1 1";

   shakeCamera = false;
   camShakeFreq = "7.0 8.0 7.0";
   camShakeAmp = "10.0 10.0 10.0";
   camShakeDuration = 0.75;
   camShakeRadius = 15.0;

   // Dynamic light
   lightStartRadius = 0;
   lightEndRadius = 0;
   lightStartColor = "0.45 0.3 0.1";
   lightEndColor = "0 0 0";

   //impulse
   impulseRadius = 0;
   impulseForce = 0;
   impulseVertical = 0;

   //radius damage
   radiusDamage        = 0;
   damageRadius        = 0;

   //burn the players?
   playerBurnTime = 5000;

};

datablock ProjectileData(skateboardDeathExplosionProjectile)
{
   directDamage        = 0;
   radiusDamage        = 0;
   damageRadius        = 0;
   explosion           = skateboardDeathExplosion;

   directDamageType  = $DamageType::jeepExplosion;
   radiusDamageType  = $DamageType::jeepExplosion;

   explodeOnDeath		= 1;

   armingDelay         = 0;
   lifetime            = 10;
};

// fucken ton of animations
datablock TSShapeConstructor(SkateboardDts)
{
	baseShape  = "./skateboard_bot.dts";
	sequence0  = "./t_root.dsq root";

	sequence1  = "./t_root.dsq run";
	sequence2  = "./t_root.dsq walk";
	sequence3  = "./t_root.dsq back";
	sequence4  = "./t_root.dsq side";

	sequence5  = "./t_root.dsq crouch";
	sequence6  = "./t_root.dsq crouchRun";
	sequence7  = "./t_root.dsq crouchBack";
	sequence8  = "./t_root.dsq crouchSide";

	sequence9  = "./t_look.dsq look";
	sequence10 = "./t_root.dsq headside";
	sequence11 = "./t_root.dsq headUp";

	sequence12 = "./t_ollie.dsq jump";
	sequence13 = "./t_ollie.dsq standjump";
	sequence14 = "./t_root.dsq fall";
	sequence15 = "./t_root.dsq land";

	sequence16 = "./t_root.dsq armAttack";
	sequence17 = "./t_root.dsq armReadyLeft";
	sequence18 = "./t_root.dsq armReadyRight";
	sequence19 = "./t_root.dsq armReadyBoth";
	sequence20 = "./t_root.dsq spearready";  
	sequence21 = "./t_root.dsq spearThrow";

	sequence22 = "./t_root.dsq talk";  

	sequence23 = "./t_root.dsq death1"; 
	
	sequence24 = "./t_root.dsq shiftUp";
	sequence25 = "./t_root.dsq shiftDown";
	sequence26 = "./t_root.dsq shiftAway";
	sequence27 = "./t_root.dsq shiftTo";
	sequence28 = "./t_root.dsq shiftLeft";
	sequence29 = "./t_root.dsq shiftRight";
	sequence30 = "./t_root.dsq rotCW";
	sequence31 = "./t_root.dsq rotCCW";

	sequence32 = "./t_root.dsq undo";
	sequence33 = "./t_root.dsq plant";

	sequence34 = "./t_root.dsq sit";

	sequence35 = "./t_root.dsq wrench";
	
	sequence36 = "./t_kickflip.dsq kickflip";
	sequence37 = "./t_shuvit.dsq shuvit";
	sequence38 = "./t_varial.dsq varial";
	sequence39 = "./t_fshuv.dsq fshuv";
	sequence40 = "./t_heelflip.dsq heelflip";
	sequence41 = "./t_heelvarial.dsq heelvarial";
	
	sequence42 = "./t_frontflip.dsq frontflip";
	sequence43 = "./t_backflip.dsq backflip";
	sequence44 = "./t_rollRight.dsq rollRight";
	sequence45 = "./t_rollLeft.dsq rollLeft";
	
	sequence46 = "./t_hardflip.dsq hardflip";
	sequence47 = "./t_inwardheel.dsq inwardheel";
};
datablock PlayerData(SkateboardPlayer)
{
   renderFirstPerson = true;
   emap = false;
   
   className = Armor;
   shapeFile = "./skateboard_bot.dts"; //dshk.dts";
   cameraMaxDist = 8;
   cameraTilt = 0.261;
   cameraVerticalOffset = 2.3;
     
   cameraDefaultFov = 90.0;
   cameraMinFov = 5.0;
   cameraMaxFov = 120.0;
   
   useEyePoint = false;
   
   
   //debrisShapeName = "~/data/player/debris_player.dts";
   //debris = horseDebris;

   aiAvoidThis = true;

   minLookAngle = -1.5708;
   maxLookAngle = 1.5708;
   maxFreelookAngle = 3.0;

   mass = 200;
   drag = 1;
   density = 5;
   maxDamage = 250;
   maxEnergy =  10;
   repairRate = 0.33;

   rechargeRate = 0.4;

   runForce = 5000;
   runEnergyDrain   = 0;
   minRunEnergy     = 0;
   maxForwardSpeed  = 100;
   maxBackwardSpeed = 50;
   maxSideSpeed     = 0;

   maxForwardCrouchSpeed  = 0;
   maxBackwardCrouchSpeed = 0;
   maxSideCrouchSpeed     = 0;

   maxUnderwaterForwardSpeed  = 0;
   maxUnderwaterBackwardSpeed = 0;
   maxUnderwaterSideSpeed     = 0;

   jumpForce       = 3000; 
   jumpEnergyDrain = 0;
   minJumpEnergy   = 0;
   jumpDelay       = 0;

   minJetEnergy   = 0;
	jetEnergyDrain = 0;
	canJet         = 0;

   minImpactSpeed   = 1;
   speedDamageScale = 3.8;

   boundingBox			= vectorScale("1.0 1.0 1.2", 4); 
   crouchBoundingBox	= vectorScale("1.0 1.0 1.2", 4);
   
   pickupRadius = 0.75;
   
   // Foot Prints
   //decalData   = HorseFootprint;
   //decalOffset = 0.25;
	
   jetEmitter        = "";
   jetGroundEmitter  = "";
   jetGroundDistance = 4;
  
   //footPuffEmitter = LightPuffEmitter;
   footPuffNumParts = 10;
   footPuffRadius = 0.25;

   //dustEmitter = LiftoffDustEmitter;

   splash = PlayerSplash;
   splashVelocity = 4.0;
   splashAngle = 67.0;
   splashFreqMod = 300.0;
   splashVelEpsilon = 0.60;
   bubbleEmitTime = 0.1;
   splashEmitter[0] = PlayerFoamDropletsEmitter;
   splashEmitter[1] = PlayerFoamEmitter;
   splashEmitter[2] = PlayerBubbleEmitter;
   mediumSplashSoundVelocity = 10.0;   
   hardSplashSoundVelocity = 20.0;   
   exitSplashSoundVelocity = 5.0;

   // Controls over slope of runnable/jumpable surfaces
   runSurfaceAngle  = 45;
   jumpSurfaceAngle = 86;

   minJumpSpeed = 20;
   maxJumpSpeed = 30;

   horizMaxSpeed = 100;
   horizResistSpeed = 33;
   horizResistFactor = 0.35;

   upMaxSpeed = 200;
   upResistSpeed = 25;
   upResistFactor = 0.3;
   
   footstepSplashHeight = 0.35;
   
   maxStepHeight = 1;

   //NOTE:  some sounds commented out until wav's are available

   JumpSound = SkateboardPopSound;

   // Footstep Sounds
   //FootSoftSound        = HorseFootFallSound;
   //FootHardSound        = HorseFootFallSound;
   //FootMetalSound       = HorseFootFallSound;
   //FootSnowSound        = HorseFootFallSound;
   //FootShallowSound     = HorseFootFallSound;
   //FootWadingSound      = HorseFootFallSound;
   //FootUnderwaterSound  = HorseFootFallSound;
   //FootBubblesSound     = FootLightBubblesSound;
   //movingBubblesSound   = ArmorMoveBubblesSound;
   //waterBreathSound     = WaterBreathMaleSound;

   //impactSoftSound      = ImpactLightSoftSound;
   //impactHardSound      = ImpactLightHardSound;
   //impactMetalSound     = ImpactLightMetalSound;
   //impactSnowSound      = ImpactLightSnowSound;
   
   //impactWaterEasy      = ImpactLightWaterEasySound;
   //impactWaterMedium    = ImpactLightWaterMediumSound;
   //impactWaterHard      = ImpactLightWaterHardSound;
   
   groundImpactMinSpeed    = 10.0;
   groundImpactShakeFreq   = "4.0 4.0 4.0";
   groundImpactShakeAmp    = "1.0 1.0 1.0";
   groundImpactShakeDuration = 0.8;
   groundImpactShakeFalloff = 10.0;
   
   //exitingWater         = ExitingWaterLightSound;

   // Inventory Items
	maxItems   = 10;	//total number of bricks you can carry
	maxWeapons = 5;		//this will be controlled by mini-game code
	maxTools = 5;
	
	uiName = "Skateboard";
	rideable = true;
   lookUpLimit = 0.5;
   lookDownLimit = 0.5;

	canRide = true;
	showEnergyBar = false;
	paintable = true;

	brickImage = horseBrickImage;	//the imageData to use for brick deployment

   numMountPoints = 1;
   mountThread[0] = "root";

   //protection for passengers
   protectPassengersBurn   = true;  //protect passengers from the burning effect of explosions?
   protectPassengersRadius = false;  //protect passengers from radius damage (explosions) ?
   protectPassengersDirect = false; //protect passengers from direct damage (bullets) ?

   useCustomPainEffects = true;
   PainHighImage = "";
   PainMidImage  = "";
   PainLowImage  = "";
   painSound     = "";
   deathSound    = "";
};

// grind emitter sparks
datablock ParticleData(SkateboardSparkParticle)
{
	dragCoefficient      = 3;
	gravityCoefficient   = 1;
	inheritedVelFactor   = 0.6;
	constantAcceleration = 0.0;
	lifetimeMS           = 850;
	lifetimeVarianceMS   = 55;
	textureName          = "base/data/particles/dot";
	spinSpeed		= 10.0;
	spinRandomMin		= -500.0;
	spinRandomMax		= 500.0;
	colors[0]     = "1.0 0.9 0.4 1.0";
	colors[1]     = "1.0 1.0 0.2 0.0";
	sizes[0]      = 0.1;
	sizes[1]      = 0.0;

	useInvAlpha = false;
};
datablock ParticleEmitterData(SkateboardSparkEmitter)
{
   ejectionPeriodMS = 10;
   periodVarianceMS = 4;
   ejectionVelocity = 8;
   velocityVariance = 4;
   ejectionOffset   = 0;
   thetaMin         = 0;
   thetaMax         = 180;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvance = false;
   particles = "SkateboardSparkParticle";

   uiName = "";
};
datablock ShapeBaseImageData(SkateboardSparkImage)
{
	shapeFile = "base/data/shapes/empty.dts";
	emap = false;

	mountPoint = 1;

	stateName[0]                  = "Ready";
	stateTransitionOnTimeout[0]   = "FireA";
	stateTimeoutValue[0]          = 0.01;
	
	stateName[1]                  = "FireA";
	stateTransitionOnTimeout[1]   = "FireC";
	stateWaitForTimeout[1]        = True;
	stateTimeoutValue[1]          = 0.0;
	stateEmitter[1]               = SkateboardSparkEmitter;
	stateEmitterTime[1]           = 0.50;
	stateSound[1]				  = "";
};

// functions for quickly mounting and unmounting the images
function Player::SkateboardMountGrindImage(%skateboard)
{
	serverPlay3D(brickPlantSound, %skateboard.getPosition());
	for(%i = 0; %i < 4; %i++)
	{
		%skateboard.wheel[%i].mountImage(SkateboardSparkImage, 0);
	}
}
function Player::SkateboardUnMountGrindImage(%skateboard)
{
	for(%i = 0; %i < 4; %i++)
	{
		if(isObject(%skateboard.wheel[%i]))
		%skateboard.wheel[%i].unmountImage(0);
	}
}

// sound updates
// only update sound when it needs to be changed
function Player::skateboardSoundUpdate(%skateboard, %speed)
{
	%soundToPlay = 1;
	if(%skateboard.isGrinding)
	{
		%soundToPlay = 2;
	}
	if(%speed < 0.2)
	{
		%soundToPlay = 0;
	}	
	if(%speed == -1)
	{
		%soundToPlay = 3;
	}
	
	if(%soundToPlay == %skateboard.skateboardSoundPlaying && %soundToPlay != 1)
		return;
		
	//%skateboard.lastSoundUpdateTime = getSimTime();
	%speedDif = (%speed - %skateboard.oldSpeed);
	%skateboard.oldSpeed = %speed;
	
		
	switch(%soundToPlay)
	{
		case 0:
			//talk("Stop sound" SPC getSimTime());
			%skateboard.skateboardSoundPlaying = 0;
			%skateboard.stopAudio(0);
		case 1:
			//talk("Roll sound" SPC getSimTime());
			%skateboard.skateboardSoundPlaying = 1;
			
			if(%speedDif > -1)
				%skateboard.playAudio(0, SkateboardRollSound);
			else
			{
				if(getSimTime() - %skateboard.lastBrakeTime > 600)
					%skateboard.playAudio(1, $SkateboardBrakeSound[getRandom(1, 3)]);
				
				%skateboard.lastBrakeTime = getSimTime();
			}
		case 2:
			//talk("Grind sound" SPC getSimTime());
			%skateboard.skateboardSoundPlaying = 2;
			%skateboard.playAudio(0, SkateboardGrindSound);
		case 3:
			%skateboard.skateboardSoundPlaying = 3;
			%skateboard.playAudio(0, SkateboardWheelSpinSound);	
		default:
			//talk("Stop sound" SPC getSimTime());
			%skateboard.skateboardSoundPlaying = 0;
			%skateboard.stopAudio(0);
	}
	
}

// logo damage in 3 stages
$SkateboardLogoMaxDam = 10;
function Player::skateboardUpdateDamage(%skateboard)
{
	%skateboard.hideNode("LogoDamage25");
	%skateboard.hideNode("LogoDamage50");
	%skateboard.hideNode("LogoDamage75");
	
	if(%skateboard.logoDamage > 25 * $SkateboardLogoMaxDam)
		%skateboard.unhideNode("LogoDamage25");
	if(%skateboard.logoDamage > 67 * $SkateboardLogoMaxDam * 2)
		%skateboard.unhideNode("LogoDamage50");
	if(%skateboard.logoDamage > 90 * $SkateboardLogoMaxDam * 3)
		%skateboard.unhideNode("LogoDamage75");
		
	if(isObject(%client = %skateboard.getMountedObject(0).client))
	{
		%client.persLogoDamage = %skateboard.logoDamage;
	}
}

// main loop. almost everything is handled in alignTOGround because that's where most of the shit happens anyway
function Player::SkateboardLoop(%obj)
{
	cancel(%obj.skateboardQuoteObject);
	%obj.skateboardAlignToGround();
	%obj.skateboardQuoteObject = %obj.schedule($Pref::Skateboard::LoopTime, SkateboardLoop);
}

// default colors, should probably make these prefs
$SkateboardDefaultWoodColor		= "0.955 0.750 0.500 1";
$SkateboardDefaultGripColor		= "0.400 0.400 0.400 1";
$SkateboardDefaultWheelColor	= "1.000 1.000 1.000 1";
$SkateboardDefaultTruckColor	= "0.700 0.700 0.700 1";
$SkateboardDefaultHardwareColor = "0.700 0.700 0.700 1";

// set color functions
function serverCmdSkateboardSetColor(%client, %part, %r, %g, %b)
{
	%skateboard = %client.player.getObjectMount();
	if(!isObject(%skateboard))
		return;

	%part = trim(strlwr(%part));
		
	switch$(%part)
	{
		case "wheels":
			%skateboard.skateboardSetWheelColor(%r SPC %g SPC %b);
			%client.skateboardWheelColor = %r SPC %g SPC %b;
		case "board":
			%skateboard.skateboardSetBoardColor(%r SPC %g SPC %b);
			%client.skateboardBoardColor = %r SPC %g SPC %b;
		case "trucks":
			%skateboard.skateboardSetTruckColor(%r SPC %g SPC %b);
			%client.skateboardTruckColor = %r SPC %g SPC %b;
		case "grip":
			%skateboard.skateboardSetGripTapeColor(%r SPC %g SPC %b);
			%client.skateboardGripColor = %r SPC %g SPC %b;
		case "hardware":
			%skateboard.skateboardSetHardwareColor(%r SPC %g SPC %b);
			%client.skateboardHardwareColor = %r SPC %g SPC %b;
	}
	
	%client.hasSetColorOnce = true;
}
function serverCmdskateboardResetColor(%client)
{
	%skateboard = %client.player.getObjectMount();
	if(!isObject(%skateboard))
		return;
		
	%client.skateboardWheelColor = $SkateboardDefaultWheelColor;
	%client.skateboardBoardColor = $SkateboardDefaultWoodColor;
	%client.skateboardTruckColor = $SkateboardDefaultTruckColor;
	%client.skateboardGripColor = $SkateboardDefaultGripColor;
	%client.skateboardHardwareColor = $SkateboardDefaultHardwareColor;
	%client.hasSetColorOnce = true;
	%skateboard.skateboardSetDefaultColor();
}

// mainly for demo purposes but i should probably restore this functionality cuz it's cool
function Player::skateboardRandomizeColors(%skateboard)
{
	%skateboard.skateboardSetGripTapeColor(getRandom(),getRandom(),getRandom());
	%skateboard.skateboardSetBoardColor(getRandom(),getRandom(),getRandom());
	%skateboard.skateboardSetWheelColor(getRandom(),getRandom(),getRandom());
	%skateboard.skateboardSetTruckColor(getRandom(),getRandom(),getRandom());
	%skateboard.skateboardSetHardwareColor(getRandom(),getRandom(),getRandom());
}
function Player::skateboardSetGripTapeColor(%skateboard, %r, %g, %b)
{
	%skateboard.setNodeColor("board_griptape", 	%r SPC %g SPC %b SPC 1);
}
function Player::skateboardSetBoardColor(%skateboard, %r, %g, %b)
{
	%skateboard.setNodeColor("board_woodDeck",		%r SPC %g SPC %b SPC 1);
	%skateboard.setNodeColor("LogoDamage25",		%r SPC %g SPC %b SPC 1);
	%skateboard.setNodeColor("LogoDamage50",		%r SPC %g SPC %b SPC 1);
	%skateboard.setNodeColor("LogoDamage75",		%r SPC %g SPC %b SPC 1);
}
function Player::skateboardSetWheelColor(%skateboard, %r, %g, %b)
{
	%skateboard.setNodeColor("wheels_front_plas",	%r SPC %g SPC %b SPC 1);
	%skateboard.setNodeColor("wheels_back_plas",	%r SPC %g SPC %b SPC 1);
}
function Player::skateboardSetTruckColor(%skateboard, %r, %g, %b)
{
	%skateboard.setNodeColor("wheels_back_truck",	%r SPC %g SPC %b SPC 1);
	%skateboard.setNodeColor("wheels_front_truck",	%r SPC %g SPC %b SPC 1);
	%skateboard.setNodeColor("board_trucks",		%r SPC %g SPC %b SPC 1);
}
function Player::skateboardSetHardwareColor(%skateboard, %r, %g, %b)
{
	%skateboard.setNodeColor("board_hardware", 		%r SPC %g SPC %b SPC 1);
}
function Player::skateboardSetDefaultColor(%skateboard)
{
	// board
	%skateboard.setNodeColor("board_griptape", 	$SkateboardDefaultGripColor);
	%skateboard.setNodeColor("board_hardware", 	$SkateboardDefaultHardwareColor);
	%skateboard.setNodeColor("board_trucks",	$SkateboardDefaultTruckColor);
	%skateboard.setNodeColor("board_woodDeck",	$SkateboardDefaultWoodColor);
	
	%skateboard.setNodeColor("LogoDamage25",	$SkateboardDefaultWoodColor);
	%skateboard.setNodeColor("LogoDamage50",	$SkateboardDefaultWoodColor);
	%skateboard.setNodeColor("LogoDamage75",	$SkateboardDefaultWoodColor);
	
	// wheels
	%skateboard.setNodeColor("wheels_front_plas",	$SkateboardDefaultWheelColor);
	%skateboard.setNodeColor("wheels_front_truck",	$SkateboardDefaultTruckColor);
	
	%skateboard.setNodeColor("wheels_back_plas",	$SkateboardDefaultWheelColor);
	%skateboard.setNodeColor("wheels_back_truck",	$SkateboardDefaultTruckColor);
}

// i forget why I need this actually, probably because im a brainlet
function Player::skateboardCheckSpawnColor(%skateboard)
{
	if(%skateboard.doNotResetColor)
		return;
		
	%skateboard.skateboardSetDefaultColor();
}

function SkateboardPlayer::onAdd(%this, %obj)
{
	Parent::onAdd(%this, %obj);
	
	// begin loop
	%obj.skateboardLoop();
	
	// set us flat
	%obj.setLookLimits(0.5, 0.5);
	
	// set a random logo
	%obj.skateboardRandomizeLogo();
	%skateboard = %obj;
	
	// hide our damage nodes so we can introduce them later
	%obj.hideNode("LogoDamage25");
	%obj.hideNode("LogoDamage50");
	%obj.hideNode("LogoDamage75");
	
	// have to set all nodes to white first otherwise we'll end up with a fucked logo
	%obj.setNodeColor("ALL", "1 1 1 1");
	%obj.schedule(0, skateboardCheckSpawnColor);
	
	// for some reason I can't mount my fucking images to the correct mount points but I can mount bots to them so we have to go that dumbass route
	%skateboard.wheel[0] = new AIPlayer()
	{
		dataBlock = emptyPlayer;
		master = %obj;
	};
	%b = %skateboard.wheel[0];
	if(isObject(%b))
	{
		MissionCleanup.add(%b);
		%b.kill();
		%obj.mountObject(%b, 1);
	}
	%skateboard.wheel[1] = new AIPlayer()
	{
		dataBlock = emptyPlayer;
		master = %obj;
	};
	%b = %skateboard.wheel[1];
	if(isObject(%b))
	{
		MissionCleanup.add(%b);
		%b.kill();
		%obj.mountObject(%b, 2);
	}
	%skateboard.wheel[2] = new AIPlayer()
	{
		dataBlock = emptyPlayer;
		master = %obj;
	};
	%b = %skateboard.wheel[2];
	if(isObject(%b))
	{
		MissionCleanup.add(%b);
		%b.kill();
		%obj.mountObject(%b, 3);
	}
	%skateboard.wheel[3] = new AIPlayer()
	{
		dataBlock = emptyPlayer;
		master = %obj;
	};
	%b = %skateboard.wheel[3];
	if(isObject(%b))
	{
		MissionCleanup.add(%b);
		%b.kill();
		%obj.mountObject(%b, 4);
	}
}

function SkateboardPlayer::onRemove(%this, %obj)
{
	// remove all 6 gorillion of my bots when the skateboard is removed
	if(isObject(%obj.wheel0))
		%obj.wheel0.delete();
	if(isObject(%obj.wheel1))
		%obj.wheel1.delete();
	if(isObject(%obj.wheel2))
		%obj.wheel2.delete();
	if(isObject(%obj.wheel3))
		%obj.wheel3.delete();
}

function SkateboardPlayer::onImpact(%this, %obj, %collidedObject, %vec, %vecLen)
{
	%skateboard = %obj;
	
	// crash us if we hit something really hard
	if(%vecLen >= 15)
	{
		%skateboard.skateboardWreck("");
		return;
	}
	
	// if not just play a cool sound
	if(%vecLen > 10)
	{
		serverPlay3D(blankaBallExplosionSound, %obj.getPosition());
	}
	else if(%vecLen > 5)
	{
		%num = getRandom(0, 2);
		switch(%num)
		{
			case 0:
			serverPlay3D(Impact2ASound, %obj.getPosition());
			case 1:
			serverPlay3D(Impact1ASound, %obj.getPosition());
			case 2:
			serverPlay3D(Impact1BSound, %obj.getPosition());
		}
	}
	
	// damaage the logo
	%obj.logoDamage += %vecLen * 0.5;
	%obj.skateboardUpdateDamage();
}

function Player::skateboardGetWheelCollides(%skateboard, %brickTop)
{
	// looks for bricks at the positions of the wheels
	// probably inefficient as FUCK but it works
	%spos = %skateboard.getPosition();
	
	// we've been passed a brick surface we're supposed to look for so use that instead of the skateboard altitude
	if(%brickTop !$= "")
	{
		%spos = "0 0" SPC %brickTop;
	}

	// find our collisions
	%collides = 0;
	for(%i = 0; %i < 4; %i++)
	{
		%skateboard.wheel[%i].colliding = false;
		%range = 0.45;
		%pos = %skateboard.wheel[%i].getPosition();
		%start = getWords(%pos, 0, 1) SPC getWord(%spos, 2) + %range;
		%end = getWords(%pos, 0, 1) SPC getWord(%spos, 2) - %range;
		%mask = $TypeMasks::FxBrickObjectType | $TypeMasks::TerrainObjectType;
		
		%raycast = containerRaycast(%start, %end, %mask);
		if(isObject(getWord(%raycast, 0)))
		{
			%collides++;
			%skateboard.wheel[%i].colliding = true;
		}
		
		%color = isObject(getWord(%raycast, 0)) ? "1 0 0 1" : "0 1 0 1";
		
		//drawLine(%start, %end, %color, 0.1); 
		//talk(%i SPC %collides SPC getSimTime());
	}
	
	// tricky, if our front or back wheels are colliding then we can boardslide, but if a pair of side wheels are colliding we can't
	// useful later when determining whether or not we should grind on ledges
	%skateboard.canBoardslide = false;
	if((!%skateboard.wheel[0].colliding && !%skateboard.wheel[1].colliding) || (!%skateboard.wheel[2].colliding && !%skateboard.wheel[3].colliding))
		%skateboard.canBoardslide = true;
	
	return %collides;
}

function Player::skateboardEndTrick(%skateboard)
{
	// finished our trick, let player release jump
	%skateboard.trickInProgress = false;
	//talk("trickdone");
}

// simple tricks
function Player::skateboardKickFlip(%skateboard)
{
	%skateboard.playThread(0, kickflip);
	%skateboard.trickInProgress = true;
	%skateboard.schedule($Pref::Skateboard::TrickTime1, skateboardEndTrick);
}
function Player::skateboardHeelFlip(%skateboard)
{
	%skateboard.playThread(0, heelflip);
	%skateboard.trickInProgress = true;
	%skateboard.schedule($Pref::Skateboard::TrickTime1, skateboardEndTrick);
}
function Player::skateboardHeelVarial(%skateboard)
{
	%skateboard.playThread(2, heelvarial);
	%skateboard.trickInProgress = true;
	%skateboard.schedule($Pref::Skateboard::TrickTime1, skateboardEndTrick);
}
function Player::skateboardShuvit(%skateboard)
{
	%skateboard.playThread(1, shuvit);
	%skateboard.trickInProgress = true;
	%skateboard.schedule($Pref::Skateboard::TrickTime1, skateboardEndTrick);
}
function Player::skateboardfShuvit(%skateboard)
{
	%skateboard.playThread(1, fshuv);
	%skateboard.trickInProgress = true;
	%skateboard.schedule($Pref::Skateboard::TrickTime1, skateboardEndTrick);
}
function Player::skateboardVarial(%skateboard)
{
	%skateboard.playThread(2, varial);
	%skateboard.trickInProgress = true;
	%skateboard.schedule($Pref::Skateboard::TrickTime1, skateboardEndTrick);
}

// flips
function Player::skateboardFlipFront(%skateboard)
{
	%skateboard.playThread(0, frontflip);
	%skateboard.trickInProgress = true;
	%skateboard.schedule($Pref::Skateboard::TrickTime3, skateboardEndTrick);
}
function Player::skateboardFlipBack(%skateboard)
{
	%skateboard.playThread(0, backflip);
	%skateboard.trickInProgress = true;
	%skateboard.schedule($Pref::Skateboard::TrickTime3, skateboardEndTrick);
}
function Player::skateboardRollRight(%skateboard)
{
	%skateboard.playThread(1, rollRight);
	%skateboard.trickInProgress = true;
	%skateboard.schedule($Pref::Skateboard::TrickTime3, skateboardEndTrick);
}
function Player::skateboardRollLeft(%skateboard)
{
	%skateboard.playThread(1, rollLeft);
	%skateboard.trickInProgress = true;
	%skateboard.schedule($Pref::Skateboard::TrickTime3, skateboardEndTrick);
}

// adv
function Player::skateboardHardflip(%skateboard)
{
	%skateboard.playThread(0, hardflip);
	%skateboard.trickInProgress = true;
	%skateboard.schedule($Pref::Skateboard::TrickTime2, skateboardEndTrick);
}
function Player::skateboardInwardHeel(%skateboard)
{
	%skateboard.playThread(0, inwardheel);
	%skateboard.trickInProgress = true;
	%skateboard.schedule($Pref::Skateboard::TrickTime2, skateboardEndTrick);
}

// interpolation of ramp alignments
// instead of snapping the limits directly to the calculated value we can increment to it over time
function Player::skateboardSmoothLookLimitTransition(%skateboard, %desiredlimit)
{
	cancel(%skateboard.skateboardLookTransitionQuota);

	if(!$Pref::Skateboard::EnableInterpol)
	{
		%skateboard.setLookLimits(%desiredlimit + $Pref::Skateboard::LookLimitSlack, %desiredlimit - $Pref::Skateboard::LookLimitSlack);
		return;
	}
	
	// average our looklimits down to one number
	%currentlimit = (getWord(%skateboard.lookLimits, 0) + getWord(%skateboard.lookLimits, 1)) / 2;
	%skateboard.setLookLimits(%currentlimit, %currentlimit);
	
	%increment = $Pref::Skateboard::InterpolIncrement;
	
	// if we're within an increments value of the desired value, just snap to it
	if(%currentlimit <= %desiredlimit + %increment && %currentlimit >= %desiredlimit - %increment)
	{
		%skateboard.setLookLimits(%desiredlimit + $Pref::Skateboard::LookLimitSlack, %desiredlimit - $Pref::Skateboard::LookLimitSlack);
		return;
	}
	
	// if not, determine the way we're supposed to be incrementing, and then increment in that direction
	if(%desiredlimit < %currentlimit)
	{
		%currentlimit -= %increment;
		%skateboard.setLookLimits(%currentlimit + $Pref::Skateboard::LookLimitSlack, %currentlimit - $Pref::Skateboard::LookLimitSlack);
	}
	else
	{
		%currentlimit += %increment;
		%skateboard.setLookLimits(%currentlimit + $Pref::Skateboard::LookLimitSlack, %currentlimit - $Pref::Skateboard::LookLimitSlack);
	}
	
	// next step
	%skateboard.skateboardLookTransitionQuota = %skateboard.schedule($Pref::Skateboard::InterpolSpeed, skateboardSmoothLookLimitTransition, %desiredlimit);
}

// converts ramp angle to a lookLimit
function skateboardAngleToLimiter(%ourAngle, %flip)
{
	%limit = %ourAngle / 180;
	%limit += 0.5;
	if(%flip)
		%limit = 1 - %limit;
	return %limit;
}

// determine if we are boardsliding. uses information gathered from wheelCollision and some information about the velocity and transform of the skateboard
// to guess whether or not we're moving perpandicular to the direction the board is facing. helpful for determining whether or not we should start a grind
// on a ledge
function Player::isBoardsliding(%skateboard)
{
	// get the direction we're moving
	%moveaxis = %skateboard.SkateboardGetDominantHorizontalVelocity();
	
	// if we're not moving back out
	if(%moveAxis $= "0")
		return -1;
		
	// get direction board is facing
	%zRot = getWord(axisToEuler(getWords(%skateboard.getTransform(), 3, 6)), 2);
	if((%zRot < -45 && %zRot > -135) || (%zRot < 135 && %zRot > 45))
	{
		%turnaxis = "x";
	}
	else
	{
		%turnaxis = "y";
	}
	
	// if we're moving in the same direction we're facing, we're not boardsliding
	if(%turnaxis $= %moveAxis)
		return false;
	
	// unless we are
	if(%turnaxis !$= %moveAxis)
		return true;
}

function Player::skateboardGrindLoop(%skateboard, %axis, %vel)
{
	// prevents multiple grind loops occuring at the same time
	cancel(%skateboard.skateboardGrindQuota);
	
	// pull us out of a grind if we're moving too slow
	if(vectorLen(%skateboard.getVelocity()) <= 1)// || !%skateboard.grindMode)
	{
		//talk("Velocity too slow");
		%skateboard.stopGrind();
		return;
	}
	
	// pull us out of the grind if too many of our wheels are on bricks
	if(%skateboard.skateboardGetWheelCollides() > 2)
	{
		//talk("Wheel cols B");
		%skateboard.stopGrind();
		return;
	}
	
	%skateboardTra = %skateboard.getTransform();
	%skateboardRot = getWords(%skateboardTra, 3, 6);
	%skateboardPos = getWords(%skateboardTra, 0, 2);
	%x = getWord(%vel, 0);
	%y = getWord(%vel, 1);
	%z = getWord(%vel, 2);
	
	//%mag = vectorLen(%vel);
	// opted for a consistent grind speed because jumping reduces a lot of our velocity anyways
	// and accounting for that results in insanely high speeds when grinding on ground-level ledges
	%mag = 10;
		
	// maintain velocity in the correct direction
	if(%axis)
	{
		%x = 0;
		%z = 0;
		
		%grindVel = vectorScale(vectorNormalize(%x SPC %y SPC %z), %mag);
		%skateboard.setVelocity(%grindVel);
	}
	else
	{
		%y = 0;
		%z = 0;
		
		%grindVel = vectorScale(vectorNormalize(%x SPC %y SPC %z), %mag);
		%skateboard.setVelocity(%grindVel);
	}
	
	// damage the logo
	%skateboard.logoDamage += 0.5;
	%skateboard.skateboardUpdateDamage();
	
	// next step
	%skateboard.skateboardGrindQuota = %skateboard.schedule($Pref::Skateboard::GrindStepTime, skateboardGrindLoop, %axis, %vel);
	return;
}

function Player::stopGrind(%skateboard)
{
	if(!%skateboard.isGrinding)
		return;

	// pull us out of a grind
	if(%skateboard.isGrinding)
		%skateboard.lastGrindStop = getSimTime();
	
	// unmount our sparks
	%skateboard.SkateboardUnMountGrindImage();
	%skateboard.isGrinding = false;
	
	// stop our grind loop
	cancel(%skateboard.skateboardGrindQuota);
	
	//sound fx
	serverPlay3D(brickPlantSound, %skateboard.getPosition());
}

function Player::commitGrind(%skateboard, %onXAxis, %center, %brickTop, %rail)
{
	// maximum amount of wheels that can be colliding with a brick during a grind
	%maxCol = 2;
	
	// too many wheels are colliding with bricks, we're probably not grinding
	%collides = %skateboard.skateboardGetWheelCollides(%brickTop);
	if(%collides > %maxCol)
	{
		return;
	}
	
	// if we're boardsliding, the front wheels or the back wheels must be free
	if(%collides > 1)
	{
		%boardsliding = %skateboard.isBoardsliding(%axis);
		if(%skateboard.canBoardslide != %boardsliding)
		{
			return;
		}
	}
	
	// some info about our skateboard right now
	%skateboardTra = %skateboard.getTransform();
	%skateboardRot = getWords(%skateboardTra, 3, 6);
	%skateboardPos = getWords(%skateboardTra, 0, 2);
	%vel = %skateboard.getVelocity();
	
	// mount our sparks image to the wheels
	%skateboard.SkateboardMountGrindImage();
	
	// snapping and passing information we need to our grind loop
	if(%onXAxis)
	{
		%x = getWord(%center, 0);
		%y = getWord(%skateboardPos, 1);
		%z = %brickTop + 0.1;
		%newPos = %x SPC %y SPC %z;
		%skateboard.setTransform(%newPos SPC %skateboardRot);
		%skateboard.skateboardGrindLoop(true, %vel);
	}
	else
	{
		%x = getWord(%skateboardPos, 0);
		%y = getWord(%center, 1);
		%z = %brickTop;
		%newPos = %x SPC %y SPC %z;
		%skateboard.setTransform(%newPos SPC %skateboardRot);
		%skateboard.skateboardGrindLoop(false, %vel);
	}
	
	%skateboard.isGrinding = true;
}

// get the direction we're moving on the horizontal plane
function Player::SkateboardGetDominantHorizontalVelocity(%skateboard)
{
	%v = %skateboard.getVelocity();
	%x = mabs(getword(%v, 0));
	%y = mabs(getword(%v, 1));
	
	if(%x < 1 && %y < 1)
		return false;
	
	if(%x > %y)
		return "x";
	else
		return "y";

}

// probably dont need this function anymore
function player::skateboardCheckCrash(%skateboard, %zDif)
{
	if(%skateboard.trickInProgress)
	{
		%skateboard.skateboardWreck(2);
	}
	return;
}

function Player::grindCheck(%skateboard, %brick, %pos, %normal)
{
	// don't snap us into a grind if we just got out of one, don't snap us into a grind if there's no brick to grind
	// don't snap us into a grind if we're not on a brick
	if(getSimTime() - %skateboard.lastGrindStop < 500 || !isObject(%brick) || %brick.getClassName() !$= "fxDTSBrick") //  || getSimTime() - %skateboard.lastJumpRelease > 500 
		return;
		
	// i forget if we actually need these values
	%x = getWord(%pos, 0);
	%y = getWord(%pos, 1);
	%z = getWord(%pos, 2);
	
	// some information about the brick
	%box = %brick.getWorldBox();
	%center = %brick.getWorldBoxCenter();
	
	// information about our skateboard
	%skatepos = %skateboard.getPosition();
	
	// are we moving in the X direction or the Y direction primarily?
	// use this to predict which way we're trying to grind
	%axis = %skateboard.SkateboardGetDominantHorizontalVelocity();
	
	// get the altitiude of the brick surface
	%edgez = getWord(%box, 5);
	%brickTop = %edgeZ;
	
	// buuurrrppp morty we have to find what edge we're on
	// need to find a better way to do this
	%edgex1 = getWord(%box, 0);
	%edgey1 = getWord(%box, 1);
	%edgex2 = getWord(%box, 3);
	%edgey2 = getWord(%box, 4);
	
	// find the lowest value, that's the edge we're most likely on
	%differenceY1 = mAbs(getWord(%skatepos, 1) - %edgeY1);
	%differenceY2 = mAbs(getWord(%skatepos, 1) - %edgeY2);
	%differenceX1 = mAbs(getWord(%skatepos, 0) - %edgeX1);
	%differenceX2 = mAbs(getWord(%skatepos, 0) - %edgeX2);
	
	if(%differenceY1 < %differenceY2)
		%dominantY = %differenceY1;
	else
		%dominantY = %differenceY2;
		
	if(%differenceX1 < %differenceX2)
		%dominantX = %differenceX1;
	else
		%dominantX = %differenceX2;
		
	if(%dominantX < %dominantY)
	{
		%edge = "x";
	}
	else
	{
		%edge = "y";
	}
	
	// the difference in altitude between our skateboard and what blockland thinks is our brick surface
	// useful for preventing fuck ups on ramps and mod-ter
	%zDifference = mAbs(mFloatLength(%brickTop - getWord(%skateboard.getPosition(), 2), 2));
	%skateboard.skateboardCheckCrash(%zDifference);
	
	// try and commit to a grind if we're moving along the edge and we're close to the z surface
	if(%axis $= "x" && %edge $= "y" && %zDifference < 0.1)
	{
		// can't grind if we're already grinding
		if(!%skateboard.isGrinding)
		{
			%skateboard.commitGrind(false, %skateboard.getPosition(), %brickTop, false);
		}
	}
	
	if(%axis $= "y" && %edge $= "x" && %zDifference < 0.1)
	{
		if(!%skateboard.isGrinding)
		{
			%skateboard.commitGrind(true, %skateboard.getPosition(), %brickTop, false);
		}
	}
}

function Player::skateboardAlignToGround(%skateboard)
{
	// find the ground below us, determine the normal of the ground, and try to adjust our skateboard accordingly.
	%zRot = getWord(axisToEuler(getWords(%skateboard.getTransform(), 3, 6)), 2);
	
	// raycast shit
	%range = 1.1;
	%mask = $TypeMasks::FxBrickObjectType |
			$TypeMasks::TerrainObjectType;
	
	%start = %skateboard.getEyePoint();
	%end = setWord(%start, 2, getWord(%start, 2) - %range);
	
	%raycast = containerRaycast(%start, %end, %mask); 
	%brick = getWord(%raycast, 0);
	if(!%brick)
	{
		// we're in the air, need to do special shit
		// lose the sound effect (unless we're doing the trick sound)
		if(!%skateboard.trickInAir)
			%skateboard.skateboardSoundUpdate(0);
			
		// cant grind on nothing lol
		%skateboard.stopGrind();
		
		// following functionality is on-ground exclusive
		return;
	}
	
	// we're not in-air
	%skateboard.trickInAir = false;
	
	// sound update for rolling on ground
	%speed = vectorLen(getWord(%skateboard.getVelocity(), 0) SPC getWord(%skateboard.getVelocity(), 1));
	%skateboard.skateboardSoundUpdate(%speed);
	
	// the actual alignment functionality. Basically we're gonna get the angle of the ramp we're on, figure out which one we're supposed to be using
	// depending on where are wheels are, and then change our looklimits accordingly. The look animation on the skateboard is set up to give the illusion
	// of being able to align to ramps. we can use the setLookLimits method to force the skateboard to look at a certain angle
	%pos = getWords(%raycast, 1, 3);
	%normal = getWords(%raycast, 4, 6);		
	%rotation = Normal2Rotation(%normal);
	%euler = axisToEuler(%rotation);
	
	%x = getWord(%euler, 0);
	%y = getWord(%euler, 1);
	%z = getWord(%euler, 2);
	
	// dont remember why i need this
	%roundedEuler = mFloatLength(%x, 1) SPC mFloatLength(%y, 1) SPC mFloatLength(%z, 1);
	
	// need to determine the angle we actually need. basically if we're parrallel with the ramp we need to be rotated, but if we're perpandicular
	// we should probasbly just be flat anyway
	if((%zRot < -45 && %zRot > -135) || (%zRot < 135 && %zRot > 45))
	{
		%ourAngle = %y;
		if(%zRot < 0)
		{
			%skateboard.skateboardSmoothLookLimitTransition(skateboardAngleToLimiter(%ourAngle, false));
		}
		else
		{
			%skateboard.skateboardSmoothLookLimitTransition(skateboardAngleToLimiter(%ourAngle, true));
		}
	}
	else
	{
		%ourAngle = %x;
		if(%zRot > -90 && %zRot < 90)
		{
			%skateboard.skateboardSmoothLookLimitTransition(skateboardAngleToLimiter(%ourAngle, true));
		}
		else
		{
			%skateboard.skateboardSmoothLookLimitTransition(skateboardAngleToLimiter(%ourAngle, false));
		}
	}
	
	// if we're on an actual brick and not the fxPlane or some gay shit we might be able to grind on it, check that
	if(%brick.getClassName() $= "fxDTSBrick")
		%skateboard.grindCheck(%brick, %pos, %normal);
}

function Player::skateboardClearDecals(%skateboard)
{
	// hide ALL logos
	for(%i = 1; %i < $SkateboardMaxLogo + 1; %i++)
	{
		%skateboard.hideNode("logo" @ %i);
	}
}
function Player::skateboardSetLogo(%skateboard, %logo)
{
	%skateboard.skateboardClearDecals();
	
	//now unhide the one we want
	%skateboard.unhideNode("logo" @ %logo);
}
function Player::skateboardRandomizeLogo(%skateboard)
{
	// set a random logo
	%skateboard.skateboardSetLogo(getRandom(1, $SkateboardMaxLogo));
}

function SkateboardPlayer::onTrigger(%this, %obj, %triggerNum, %val)
{
	Parent::onTrigger(%this, %obj, %triggerNum, %val);
		
	// jumping function
	// set Trickmode to the value assocaited with whether or not we're holding jump
	%skateboard = %obj;
	
	// we just released the jump key mid-trick, crash
	if(%skateboard.trickInProgress)
	{
		%skateboard.skateboardWreck(1);
	}
	%skateboard.lastJumpTime = getSimTime();
	%skateboard.trickMode = %val;
	
	// jumping takes us out of a grind
	%skateboard.stopGrind();
	
	// jumping damages the logo as well
	%obj.logoDamage += %val;
	%obj.skateboardUpdateDamage();
	//talk(%skateboard.trickMode);
}

function Player::skateboardWreck(%obj, %msg)
{
	// we're not a skateboard, abort
	if(%obj.getDatablock() != SkateboardPlayer.getID())
		return;
	
	// we've already wrecked, abort
	if(getSimTime() - %obj.lastWreckTime < 2000)
		return;
		
	// can't land poorly if we're not actually landing
	if(%msg == 2 && (getSimTime() - %obj.lastJumpTime) < 200)
		return;
		
	// wrecked variables
	%obj.isWrecked = true;
	%obj.lastWreckTime = getSimTime();
	
	// get our player
	%player = %obj.getMountedObject(0);
	if(!isObject(%player))
	{
		return;
	}
	
	// display reason for crash
	switch(%msg)
	{
		case 1:
			%msgText = "Don't release the jump key until the trick is done!";
		case 2:
			%msgText = "Bad landing!";
		default:
			%msgText = "";
	}
	
	%msgText = "\c3" @ %msgText;
	
	if(isObject(%player.client) && %msgText != -1)
		centerPrint(%player.client, %msgText, 2);
	
	// play random pain sound
	%in = getRandom(0, 1);
	if(%in)
		serverPlay3D(PainCrySound, %player.getPosition());
	else
		serverPlay3D(deathCrySound, %player.getPosition());

	// stop grinding at this point if we are
	%obj.stopGrind();
	
	// tumble taken from Skis
	%speed = vectorLen(%obj.getVelocity());
	%time = ( ( (%speed-10) / 50) * 7) + 1;
	%time = %time * 1000;
	if(%time > 7000)
		%time = 7000;
	if(%time < 1000)
		%time = 1000;
	%player.canDismount = false;
	%player.setWhiteout(%time/7000 * 1);
	tumble(%player, %time);
	return;
}

function Player::skateboardDoDeath(%skateboard)
{
	// spawn debris and get rid of the board
	%p = new Projectile()
	{
		dataBlock = skateboardDeathExplosionProjectile;
		initialPosition = %skateboard.getPosition();
		initialVelocity = "0 0 1";
	};
	MissionCleanup.add(%p);
	%skateboard.delete();
}

// list of our boards and their descriptions
// makes it easier to add new designs
$skateboardLogoIndex = 0;
$skateboardLogo[$skateboardLogoIndex++] = "ATM Click 1";
$skateboardLogo[$skateboardLogoIndex++] = "ATM Click 2";
$skateboardLogo[$skateboardLogoIndex++] = "ATM Click 3";
$skateboardLogo[$skateboardLogoIndex++] = "ATM Click 4";
$skateboardLogo[$skateboardLogoIndex++] = "IFO 1";
$skateboardLogo[$skateboardLogoIndex++] = "IFO 2";
$skateboardLogo[$skateboardLogoIndex++] = "IFO 3";
$skateboardLogo[$skateboardLogoIndex++] = "IFO 4";
$skateboardLogo[$skateboardLogoIndex++] = "Birdhouse";
$skateboardLogo[$skateboardLogoIndex++] = "Zoo York";
$skateboardLogo[$skateboardLogoIndex++] = "Thrasher";
$skateboardLogo[$skateboardLogoIndex++] = "Girl Malto";
$skateboardLogo[$skateboardLogoIndex++] = "Baker";
$skateboardLogo[$skateboardLogoIndex++] = "Zero";
$skateboardLogo[$skateboardLogoIndex++] = "Blockland";
$skateboardLogo[$skateboardLogoIndex++] = "Nigeria";
$SkateboardMaxLogo = $skateboardLogoIndex;

// function for changing logo
function serverCmdSkateboardLogo(%client, %num)
{
	%board = %client.player.getObjectMount();
	if(%board.getDatablock() == SkateboardPlayer.getID())
	{
		if(%num < 1 || %num > $SkateboardMaxLogo)
		{
			messageClient(%client, '', "Logo does not exist! Type /skateboardLogoHelp for a list of all logos.");
			return;
		}
		%board.skateboardSetLogo(%num);
		%client.prefSkateboardIndex = %num;
		
		// also removes all damage from the logo
		%board.logoDamage = 0;
		%board.skateboardUpdateDamage();
	}
}
function serverCmdSkateboardLogoHelp(%client)
{
	// displays all avalible logos
	messageClient(%client, '', "\c3-------------------");
	messageClient(%client, '', "\c3Index - Description");
	for(%i = 1; %i <= $SkateboardMaxLogo; %i++)
	{
		messageClient(%client, '', "\c5" @ %i SPC "-\c4" SPC $skateboardLogo[%i]);
	}
	messageClient(%client, '', "\c3-------------------");
}
function serverCmdSkateboardHelp(%client)
{
	messageClient(%client, '', "\c3/skateboardLogo (index) \c6- Sets the skateboard's logo to the specified logo.");
	messageClient(%client, '', "\c3/skateboardLogoHelp \c6- Lists all logo indexes along with their logo names.");
	messageClient(%client, '', "\c3/skateboardResetColor \c6- Resets the skateboard's color back to it's default values.");
	messageClient(%client, '', "\c3/skateboardSetColor (Wheels/Trucks/Board/Grip/Hardware) (R G B) \c6- Sets the specified part to the specified RGB value.");
	
	if($Pref::Skateboard::EnableSessionMarkers)
	messageClient(%client, '', "\c3/skateboardMarker [Set] \c6- Teleports you to your set marker, or sets your marker at your current position.");
}

// session marker from skate 3
function serverCmdSkateboardMarker(%client, %string)
{
	if(!$Pref::Skateboard::EnableSessionMarkers)
		return;

	%skateboard = %client.player.getObjectMount();
	
	%stringV = strlwr(trim(%string));
	
	if(%stringV $= "set")
	{
		if(isObject(%skateboard))
		{
			%client.currentSessionMarker = %skateboard.getTransform();
			%t = %client.currentSessionMarker;
			%x = mFloatLength(getWord(%t, 0), 0);
			%y = mFloatLength(getWord(%t, 1), 0);
			%z = mFloatLength(getWord(%t, 2), 0);
			%t = %x SPC %y SPC %z;
			messageClient(%client, '', "\c2Session marker set to position\c3" SPC %t);
		}
		else
		{
			messageClient(%client, '', "You must be on a skateboard to use this command.");
		}
	}
	else
	{
		if(%client.currentSessionMarker !$= "")
		{
			// spawns a new player for the client, by extension pulling them out of any vehicles.
			// useful because it allows the player to cut a tumble short without ctrl-k
			%client.spawnPlayer();
			%player = %client.player;
			%player.setTransform(%client.currentSessionMarker);
			%player.setWhiteout(0.4);
			
			// put them back on their board so they can get right back to skating instantly
			%player.startSkateboarding();
			commandToClient(%client, 'setScrollMode', -1);
			%player.unMountimage(%slot);
			fixArmReady(%player);
		}
		else
		{
			messageClient(%client, '', "You don't have a session marker. Type /skateboardMarker Set");
		}
	}
}

package skateboardPackage
{
	function Armor::onCollision(%this, %obj, %col, %vec, %speed)
	{
		// weird issue where if a skateboard rolls over an item, it'll pick it up (but not really)
		// causes a fuckup in the rider's inventory, so just disable all collisions with items
		if(%this == skateboardPlayer.getID())
		{
			if(%obj.getClassName() $= "Item" || %col.getClassName() $= "Item")
			{
				return;
			}
		}
		Parent::onCollision(%this, %obj, %col, %vec, %speed);
	}

	// functionality for tricks
	function serverCmdShiftBrick(%client, %x, %y, %z)
	{
		Parent::serverCmdShiftBrick(%client, %x, %y, %z);
		
		%player = %client.player;
		if(!isObject(%player))
			return;
		
		%mount = %player.getObjectMount();
		if(!isObject(%mount))
			return;
			
		if(%mount.getDatablock() == SkateboardPlayer.getID())
		{
			%skateboard = %mount;
			if(%skateboard.trickMode)
			{
				%trickVector = %x SPC %y SPC %z;
				//talk("N" SPC %trickVector);
				
				switch$ (%trickVector)
				{
					case "0 -1 0":
						%skateboard.skateboardKickFlip();
					case "0 1 0":
						%skateboard.skateboardShuvIt();
					case "0 0 -3":
						%skateboard.skateboardVarial();
					case "1 0 0":
						%skateboard.skateboardFlipFront();
					case "0 0 -1":
						%skateboard.skateboardHardflip();
					case "0 0 1":
						%skateboard.skateboardInwardHeel();
					default:
						return;
				}
				
				// play the grip scrape sound
				serverPlay3D(SkateboardTrickSound, %skateboard.getPosition());
				
				// wheels spinning sound
				%skateboard.skateboardSoundUpdate(-1);
				
				// useful for sound functionality. if this is false, the skateboard will stop any audio when it's in the air
				%skateboard.trickInAir = true;
				
				// can't trick instantly after doing one trick
				%skateboard.trickMode = false;
			}
		}
	}
	
	function serverCmdSuperShiftBrick(%client, %x, %y, %z)
	{
		Parent::serverCmdSuperShiftBrick(%client, %x, %y, %z);
		
		%player = %client.player;
		if(!isObject(%player))
			return;
		
		%mount = %player.getObjectMount();
		if(!isObject(%mount))
			return;
			
		if(%mount.getDatablock() == SkateboardPlayer.getID())
		{
			%skateboard = %mount;
			if(%skateboard.trickMode)
			{
				%trickVector = %x SPC %y SPC %z;
				
				switch$ (%trickVector)
				{
					case "0 -1 0":
						%skateboard.skateboardHeelFlip();
					case "0 1 0":
						%skateboard.skateboardFShuvIt();
					case "0 0 -1":
						%skateboard.skateboardHeelVarial();
					case "1 0 0":
						%skateboard.skateboardFlipBack();
					default:
						return;
				}
				
				serverPlay3D(SkateboardTrickSound, %skateboard.getPosition());
				%skateboard.skateboardSoundUpdate(-1);
				%skateboard.trickInAir = true;
				%skateboard.trickMode = false;
			}
		}
	}
	
	function ServerCmdRotateBrick(%client, %dir)
	{
		Parent::ServerCmdRotateBrick(%client, %dir);
		
		%player = %client.player;
		if(!isObject(%player))
			return;
		
		%mount = %player.getObjectMount();
		if(!isObject(%mount))
			return;
			
		if(%mount.getDatablock() == SkateboardPlayer.getID())
		{
			%skateboard = %mount;
			if(%skateboard.trickMode)
			{
				if(%dir < 0)
				{
					%skateboard.skateboardRollLeft();
				}
				else
				{
					%skateboard.skateboardRollRight();				
				}
				
				serverPlay3D(SkateboardTrickSound, %skateboard.getPosition());
				%skateboard.skateboardSoundUpdate(-1);
				%skateboard.trickInAir = true;
				%skateboard.trickMode = false;
			}
		}
	}
	
	// function Armor::onTrigger(%this, %obj, %triggerNum, %val)
	// {
		// // %skateboard = %obj.getObjectMount();
		
		// // if(isObject(%skateboard))
		// // {
			// // if(%skateboard.getDatablock() == SkateboardPlayer.getID())
			// // {
				// // if(!%triggerNum && %val)
				// // {
					// // %sv = %skateboard.getVelocity();
					// // if(vectorLen(%sv) < 20)
					// // {	
						// // talk(vectorLen(%sv));
						// // %vec = vectorAdd(%sv, vectorScale(%skateboard.getEyeVector(), 15));
						// // %vec = getWord(%vec, 0) SPC getWord(%vec, 1) SPC getWord(%sv, 2) + 1;
						// // %skateboard.setVelocity(%vec);
					// // }
				// // }
			// // }
		// // }
		// Parent::onTrigger(%this, %obj, %triggerNum, %val);
	// }
	
	// adds an extra field that allows us to easily determine what the player's look limits are at any time
	function Player::setLookLimits(%player, %up, %down)
	{
		Parent::setLookLimits(%player, %up, %down);
		%player.lookLimits = %up SPC %down;
	}
	
	// since the wheelbots are killed on spawn, we don't want them to despawn instantly
	// also extends skateboard's death functionality, opting for our special effects one instead
	// of the default white poof
	function armor::onDisabled(%db,%pl,%bool)
	{
		if(%db.isEmptyPlayer)
			return;
		if(%db == SkateboardPlayer.getID())
		{
			%pl.schedule(10, skateboardDoDeath);
			return;
		}
		return parent::onDisabled(%db,%pl,%bool);
	}
	
	// function PrintGunImage::onHitObject(%this, %player, %slot, %hitObj, %hitPos, %hitNormal)
	// {
		// %radius = 0.1;
		// %mask = $TypeMasks::PlayerObjectType;
		// initContainerRadiusSearch(%hitPos, %radius, %mask);
		// while(%object = containerSearchNext())
		// {
			// %dataName = %object.getDatablock().getName();
			
			// if(%dataName !$= "SkateboardPlayer" SPC getSimTime())
				// continue;
				
			// break;
			
		// }

		// commandToClient(%player.client, 'clientCmdOpenPrintSelectorDlg', "1x1", 0, 3);
		// Parent::onHitObject(%this, %player, %slot, %hitObj, %hitPos, %hitNormal);
	// }
};
activatePackage(skateboardPackage);